#!/bin/bash

# luckyBackup.sh

IFS=$',' 		# set IFS to treat spaces as spaces - DO NOT touch this


# user input start----------------------------------------------------------------------------------------------
# Sync ---------------------------------------------------------------------------------------------------------
# Please define directories/files to be synced first :

syncdirA[0]="/media/LUCKYUSB/diplomatiki/"
syncdirB[0]="/home/luckyb/UCL/TBC/"

#syncdirA[1]="path/to/2nd/syncdirA"
#syncdirB[1]="path/to/2nd/syncdirB/"
# 


# Backup -------------------------------------------------------------------------------------------------------
# Please define directories to be backed up as well as the destination of the backups :

sourcedir[0]="/home/luckyb/"
destdir[0]="/media/backups/linux/homeMAIN/"

sourcedir[1]="/etc/"
destdir[1]="/media/backups/linux/etc/"

sourcedir[2]="/mnt/OS2/home/luckyb/"
destdir[2]="/media/backups/linux/homeOS2/"

sourcedir[3]="/media/xorao ta panta/mp3/"
destdir[3]="/media/backups/mp3/"

sourcedir[4]="/media/xorao ta panta/photos"
destdir[4]="/media/backups/Photos/"

sourcedir[5]="/mnt/winXPpc/goodika/"
destdir[5]="/media/backups/winXP/goodika Documents/"

#sourcedir[6]="/path/2/other/source directory/"
#destdir[6]="/path/2/other/destination directory/"


# sourcedir[x] is the source directory and destdir[x] is the destination backup directory
# 'x' is an integer starting from 0 and increasing by 1. It must be the same for the sourcedir and the destdir
# use '/' at the end of the sourcedir and destdir not to copy 'root' source but only subfolders and files

# Example ---------------------------------------------------------------------------------------------------------
# The following will backup /home/luckyb/mp3 to /media/backups/ - a directory /media/backups/mp3 will be Created
# The directory /home/luckyb/photos will be backed up to /media/backups/luckyb_photos - All contents of the /home/luckyb/photos directory will be backedup but no photos directory will be created in /media/backups/luckyb_photos

# sourcedir[0]="/home/luckyb/mp3"
# destdir[0]="/media/backups"
#
# sourcedir[1]="/home/luckyb/photos/"
# destdir[1]="/media/backups/luckyb_photos/"


# user input end-----------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------

# color variables
W="[1;31m" # red - Warning
E="[1;33m" # yellow - info
S="[1;32m" # green - info 2
Q="[1;36m" # CYAN - User Action
M="[1;34m" # BLUE - Info 3
C="[1;35m" # MAGENTA - data
N="[0;39m" # default system console color: Normal


#--------------------------------------------------------------------------------------------------------------
# main script (this is where all is done)
# Do not modify this part unless you really know what you are doing (You have been warned)

# Introductory information ------------------------------------------------------------------------------------
intro()
{
clear
echo
echo "${M}luckybackup script  -  version 0.1"
echo "${S}This script will (hopefully) sync and backup everything you need in no time${N}"
echo "(Well, it might take a little longer if run for the first time ;-)"
echo
echo "${E}Make sure that:"
echo "You have already declared and mounted all the directories that need to be synced/backed-up"
echo "If you haven't, don't worry. I will check everything before doing something  stupid !!"
echo
echo "Hit ${Q}'enter' to continue${E} otherwise press ${Q}'ctrl+c' to exit NOW${N}"
read
echo
}

# Check if the script is run with su privileges and ask for confirmation ----------------------------------
check_user()
{
declare -a options
options[${#options[*]}]="Just continue with the script, I know what I'm doing"
options[${#options[*]}]="Quit now. I'll try again as su"

if [ "$USER" != "root" ]
then
	echo
	echo "${S}$USER,"
	echo "${W}Having in mind that ${C}you are not root${W}, some of the operations you have asked for, might produce errors"
	echo
	echo "${Q}What would you like me to do??${N}"
	select opt in "${options[@]}";
	do
  		case ${opt} in
  			${options[0]})
				echo "${E}ok, but you have been WANRED${N}"; ;;
			${options[1]})
				echo "${E}exiting now...${N}";
				exit 0; ;;
  		esac;
		break
	done

fi
}

# Check if the directories declared are valid and ask for confirmation to go on -----------------------------
check_dirs()
{
clear
echo "${S}...Checking if the directories you specified exist or are empty${N}"
echo
sleep 1
checkdirs=( ${syncdirA[*]} ${syncdirB[*]} ${sourcedir[*]} ${destdir[*]}) #array containing all dirs to be checked

for check in ${checkdirs[@]}
do
	if [ -d $check ] # does directory exist ??
	then
		if [ "$(ls -A $check)" ]
		then
#	     		echo "$check: ok"
			false
		else
    			echo "${W}WARNING:${N} Directory ${W}$check${N} is Empty (Check if you mount something there !!)"
			emptydirs[${#emptydirs[*]}]=$check		#array containing all empty dirs
		fi
	else
		echo "${W}WARNING:${N} Directory ${W}$check${N} does NOT exist"
		wrongdirs[${#wrongdirs[*]}]=$check		#array containing all non existent dirs
	fi
done

if [ ${#wrongdirs[*]} != 0 -o ${#emptydirs[*]} != 0 ]
then
	declare -a options
	options[${#options[*]}]="Just continue with the script, I know what I'm doing !!"
	options[${#options[*]}]="Quit now. I'll fix the problems and try again"
	echo
	echo "${W}Total: ${#wrongdirs[*]} non existent and ${#emptydirs[*]} empty directories${N}"
	echo "${E}It is advised that you quit now and fix any problems"
	echo "To keep your data safe I will:"
	echo "-> Skip syncing if one or both of the sync dirs do not exist"
	echo "-> Create any backup directories if they don't exist"
	echo "   (which might be what you want or you might have mistyped the backup dir name)"
	echo "-> Skip backing up of empty or not mounted source directories"
	echo 
	echo "${Q}What would you like me to do??${N}"
	select opt in "${options[@]}";
	do
  		case ${opt} in
  			${options[0]})
				echo "${E}ok, continuing with the script...${N}";
				sleep 1; ;;
			${options[1]})
				echo "${E}exiting now...${N}";
				exit 0; ;;
  		esac;
		break
	done
else
	echo "${S}...All directories valid :-)${N}"
fi
}

#This is where syncing is performed ---------------------------------------------------------------
sync_stuff()
{
echo "${S} ------------------------------- Syncing stuff -------------------------------  ${N}"
echo

syncno=${#syncdirA[*]}		#No. of dirs to be synced
count=0				#typical count variable
while [ $count -le $((syncno-1)) ]
do
	echo "now syncing directory $((count+1)) of $syncno:"
	echo "${C} ${syncdirA[$count]} ${N}"
	echo "with"
	echo "${C} ${syncdirB[$count]} ${N}"
	if [ -d ${syncdirA[$count]} ] && [ -d ${syncdirB[$count]} ]	#check if both sync directories exist
	then
#		echo "Executing rsync NOW !!"
		rsync --archive --progress --stats --update -h ${syncdirB[$count]} ${syncdirA[$count]}
		rsync --archive --progress --stats --update -h ${syncdirA[$count]} ${syncdirB[$count]}
		echo "...done"
		echo
	else
		echo "${W}One or both directories are missing...${N}" 
		echo "...skipping"
		echo
	fi
	count=$((count+1))
done
}

#This is where backing-up is performed ---------------------------------------------------------------
backup_stuff()
{

dirno=${#sourcedir[*]}		#No. of dirs to be backed up

echo "${S} ------------------------------- Backing up stuff -------------------------------  ${N}"
echo
count=0
while [ $count -le $((dirno-1)) ]
do
	echo "backing up directory $((count+1)) of $dirno:"
	echo "${C} ${sourcedir[$count]} ${N}"
	echo "to"
	echo "${C} ${destdir[$count]} ${N}"
	if [ -d ${sourcedir[$count]} ] && [ "$(ls -A ${sourcedir[$count]})" ]	#check if sourcedir exists and is not empty
	then
#		echo "Executing rsync NOW !!"
		rsync -h --archive --progress --stats --update --delete-after  ${sourcedir[$count]} ${destdir[$count]}
		echo "...done"
		echo
	else
		echo "${W}Directory ${sourcedir[$count]} is missing${N}" 
		echo "...skipping"
		echo
	fi
	count=$((count+1))

done
}

thats_all()
{
echo
echo "${Q}Syncing and backing up is finished"
echo "hope everything went ok :-)${N}"
echo
}


# main code --------------------------------------------------------------------------------------

intro
check_user
check_dirs
sync_stuff
backup_stuff
thats_all

unset IFS # IFS back to normal
exit 0
